from rest_framework import viewsets
from rest_framework import status
from rest_framework.decorators import api_view
from rest_framework.response import Response
from django_filters.rest_framework import DjangoFilterBackend
from django.contrib.auth.models import User
from on_demand.models import Connection, ConsumerProfile, SupplierProfile
from on_demand.serializers import ConnectionSerializer, ConsumerProfileSerializer, SupplierProfileSerializer
from on_demand.signals import connection_requested, connection_finished, connection_cancelled, connection_accepted, connection_rejected
from django.conf import settings


class ConnectionViewSet(viewsets.ModelViewSet):
    queryset = Connection.objects.all()
    serializer_class = ConnectionSerializer
    filter_backends = (DjangoFilterBackend,)
    filter_fields = ('id', 'supplier_id', 'consumer_id', 'status')

    def create(self, request, *args, **kwargs):
        objective = request.data['objective']
        if ('consumer_request_comments' in request.data.keys()):
            consumer_request_comments = request.data['consumer_request_comments']
        else:
            consumer_request_comments = None

        if ('connection_status' in request.data.keys()):
            connection_status = request.data['connection_status']
        else:
            connection_status = "STARTED"

        supplier_id = request.data['supplier_id']
        consumer_id = request.data['consumer_id']
        consumer = User.objects.get(id=consumer_id)
        supplier = User.objects.get(id=supplier_id)
        serializer = ConnectionSerializer(data={
            'consumer': consumer.id,
            'supplier': supplier.id,
            'objective': objective,
            'status': connection_status,
            'consumer_request_comments': consumer_request_comments})
        serializer.is_valid()
        connection_created = serializer.save()
        if(connection_created is True):
            connection_requested.send(
                sender=self.__class__, from_user=consumer.profile.email, to_user=supplier.profile.email)
        return Response(serializer.data, status=status.HTTP_201_CREATED)

    def update(self, request, *args, **kwargs):
        connection_id = request.data['connectionId']
        connection_status = request.data['status']
        role = request.data['role']
        connection = Connection.objects.get(id=connection_id)
        connection.status = connection_status
        role_to_notify, user = ('supplier', connection.supplier) if role == 'consumer' else (
            'consumer', connection.consumer)
        from_user = connection.consumer if role == 'consumer' else connection.supplier
        if(connection_status == 'cancelled'):
            connection_cancelled.send(sender=self.__class__, from_user=from_user.profile.email,
                                      to_user=user.profile.email, role_to_notify=role_to_notify, connection_cancelled=connection)
        elif(connection_status == 'finished'):
            ranking = request.data['ranking']
            connection.ranking = ranking
            connection_finished.send(sender=self.__class__, from_user=from_user.profile.email,
                                     to_user=user.profile.email, connection=connection, supplier_ranking=ranking)
        elif(connection_status == 'ongoing'):
            connection_accepted.send(sender=self.__class__, from_user=from_user.profile.email,
                                     to_user=user.profile.email, connection=connection)
        elif(connection_status == 'rejected'):
            rejection_reason = request.data['rejection_reason']
            connection.rejection_reason = rejection_reason
            connection_rejected.send(sender=self.__class__, from_user=from_user.profile.email,
                                     to_user=user.profile.email, connection=connection, rejection_reason=rejection_reason)

        connection.save()
        return Response(status=status.HTTP_200_OK)



from django.conf.urls import url, include
from rest_framework import routers
from on_demand.views import (status_views,
                             role_view,
                             supplier_profile_views,
                             consumer_profile_views,
                             search_views,
                             connection_views)

router = routers.DefaultRouter(trailing_slash=False)
router.register('connection', connection_views.ConnectionViewSet)

urlpatterns = [
    url('status/', status_views.status),
    url('users/change_role/', role_view.change_role),
    url(r'^users/(?P<user_id>[0-9]+)/supplier_profile',
        supplier_profile_views.supplier_profile),
    url(r'^users/(?P<user_id>[0-9]+)/consumer_profile',
        consumer_profile_views.consumer_profile),
    url('newest-suppliers', search_views.newest_suppliers),
    url(r'^find-suppliers/$', search_views.find_suppliers),
    url('', include(router.urls))
]



from rest_framework.decorators import api_view, permission_classes
from django.db import transaction
from django.contrib.auth.models import User
from rest_framework import status
from rest_framework.decorators import action
from rest_framework.response import Response
from rest_framework.permissions import AllowAny
import on_demand.services.profile_service as profile_service
from on_demand.serializers import UserSerializer, SupplierProfileSerializer, ConsumerProfileSerializer


@api_view(['POST'])
@transaction.atomic
@permission_classes((AllowAny, ))
def change_role(request):
    body = request.data
    user_id = body['user_id']
    user_type = body['user_type']

    user = User.objects.get(id=user_id)
    profile = profile_service.create_profile(user, user_type)
    if (user_type == 'supplier'):
        serializer = SupplierProfileSerializer(profile)
    else:
        serializer = ConsumerProfileSerializer(profile)
    return Response(serializer.data, status=status.HTTP_200_OK)



from django.shortcuts import render
from rest_framework.decorators import api_view, permission_classes
from rest_framework.response import Response
from rest_framework.permissions import AllowAny

@api_view(['GET'])
@permission_classes((AllowAny, ))
def status(request):
    return Response({'status': 'up'})


from django.test import TestCase
from django.urls import reverse


class NewestSuppliersTest(TestCase):
    def test_newest_suppliers_url_exists_at_desired_location(self):
        response = self.client.get('/api/newest-suppliers/')
        self.assertEqual(response.status_code, 200)

    def test_newest_suppliers_returns_empty_array_when_no_suppliers(self):
        response = self.client.get('/api/newest-suppliers/')
        self.assertEqual(response.status_code, 200)
        self.assertListEqual([], response.json())



# Generated by Django 2.1 on 2020-05-26 16:58
from django.db import migrations

# Running FULL TEXT migrations only if databse engine is mysql
# We should later find the syntax for FULL TEXT capabilities in other popular engines
def forwards(apps, schema_editor):
  if not schema_editor.connection.vendor.startswith('mysql'):
      print('Database vendor: {}'.format(schema_editor.connection.vendor))
      print('Skipping migration without attempting to ADD FULL TEXT SEARCH')
      return

  migrations.RunSQL('ALTER TABLE on_demand_userdetails ADD FULLTEXT (description)')

class Migration(migrations.Migration):

    dependencies = [
        ('on_demand', '0001_initial'),
    ]

    operations = [
        migrations.RunPython(forwards)
    ]



from rest_framework import filters
from rest_framework import viewsets
from rest_framework import status
from rest_framework.decorators import api_view
from rest_framework.response import Response
from django.contrib.auth.models import User
from on_demand.models import SupplierProfile
from on_demand.serializers import SupplierProfileSerializer, UserSerializer
import urllib.parse


@api_view(['GET'])
def newest_suppliers(request):
    """
    Retrieve the five newest suppliers that joined, nesting user data
    """
    supplier_profiles = SupplierProfile.objects.select_related('user').filter(user__first_name__isnull=False, user__last_name__isnull=False,
                                                                              user__details__description__isnull=False).order_by('-date_joined')[:30]
    supplier_serializer = SupplierProfileSerializer(
        supplier_profiles, many=True)
    return Response(supplier_serializer.data)


@api_view(['GET'])
def find_suppliers(request):
    search_term = request.GET.get('search_term')
    decoded_search_term = urllib.parse.unquote(search_term)
    search_query_tables = "SELECT * FROM api_supplier_profile M LEFT JOIN auth_user U ON M.user_id = U.id LEFT JOIN api_userdetails P ON P.user_id = U.id WHERE "
    search_query_names_like = "U.first_name LIKE %s OR U.last_name LIKE %s OR "
    search_query_full_text_profile = "MATCH (P.description) AGAINST (%s IN NATURAL LANGUAGE MODE)"
    complete_search_query = search_query_tables + search_query_names_like + \
        search_query_full_text_profile + search_query_full_text_supplier_profile
    results = SupplierProfile.objects.raw(
        complete_search_query, ['java', 'java', 'java', 'java', decoded_search_term])
    profile_serializer = SupplierProfileSerializer(results, many=True)
    return Response(profile_serializer.data)



from rest_framework import status
from rest_framework.decorators import api_view
from rest_framework.response import Response
from on_demand.models import SupplierProfile
from on_demand.serializers import SupplierProfileSerializer


@api_view(['GET', 'PUT'])
def supplier_profile(request, user_id):
    """
    Retrieve, update or delete a code snippet.
    """
    try:
        supplier_profile = SupplierProfile.objects.get(user_id=user_id)
    except SupplierProfile.DoesNotExist:
        return Response(status=status.HTTP_404_NOT_FOUND)

    if request.method == 'GET':
        supplier_profile = SupplierProfileSerializer(supplier_profile)
        return Response(supplier_profile.data)

    elif request.method == 'PUT':
        serializer = SupplierProfileSerializer(
            supplier_profile, data=request.data)
        if serializer.is_valid():
            serializer.save()
            return Response(serializer.data)
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)



from django.apps import AppConfig
from django.core.checks import register

from . import settings_validation


class OnDemandConfig(AppConfig):
    name = 'on_demand'

    def ready(self):
        import on_demand.receivers
        register(
            settings_validation.required_installed_apps
        )




from django.apps import apps
from django.core.checks import Error

REQUIRED_INSTALLED_APPS = [
    'rest_framework'
]


def required_installed_apps(app_configs, **kwargs):
    errors = []
    for app in REQUIRED_INSTALLED_APPS:
        if not apps.is_installed(app):
            errors.append(
                Error(f'{app} is required in INSTALLED_APPS')
            )
    return errors



from django.contrib.auth.models import User
from on_demand.models import ConsumerProfile, SupplierProfile
from on_demand.serializers import UserSerializer, SupplierProfileSerializer, ConsumerProfileSerializer


def create_profile(user, user_type):
    if user_type == 'supplier':
        profile, created = SupplierProfile.objects.get_or_create(user=user)
    else:
        profile, created = ConsumerProfile.objects.get_or_create(user=user)
    return profile



from django.contrib.auth.models import User
from on_demand.models import UserDetails, SupplierProfile, ConsumerProfile, Connection
from rest_framework import serializers


class UserDetailsSerializer(serializers.ModelSerializer):
    class Meta:
        model = UserDetails
        fields = ('email', 'picture', 'linkedin', 'behance', 'twitter',
                  'instagram', 'facebook', 'youtube', 'description')


class UserSerializer(serializers.ModelSerializer):
    userDetails = UserDetailsSerializer(read_only=True)

    class Meta:
        model = User
        fields = ('id', 'username', 'userDetails')


class UserSerializerWithoutAuthData(serializers.ModelSerializer):
    userDetails = UserDetailsSerializer(read_only=True)

    class Meta:
        model = User
        fields = ('id', 'username', 'userDetails')


class SupplierProfileSerializer(serializers.ModelSerializer):
    user = UserSerializer(read_only=True)

    class Meta:
        model = SupplierProfile
        fields = ('user_id', 'user',
                  'finished_connections_count', 'connections_ranking_accumulator')


class ConsumerProfileSerializer(serializers.ModelSerializer):
    user = UserSerializer(read_only=True)

    class Meta:
        model = ConsumerProfile
        fields = ('user_id', 'user')


class ConnectionSerializer(serializers.ModelSerializer):
    class Meta:
        model = Connection
        fields = ('id',
                  'supplier',
                  'consumer',
                  'status',
                  'date_created',
                  'date_finished',
                  'objective',
                  'rejection_reason',
                  'consumer_request_comments',
                  'finish_reason',
                  'ranking')

    def create(self, validated_data):
        return Connection.objects.create(**validated_data)



from on_demand.models import UserDetails
from django.contrib.auth.models import User
from django.db.models.signals import post_save
from django.dispatch import receiver


@receiver(post_save, sender=User)
def create_user_profile(sender, instance, created, **kwargs):
    if created:
        UserDetails.objects.create(user=instance, email=instance.username)


@receiver(post_save, sender=User)
def save_user_profile(sender, instance, **kwargs):
    instance.details.save()



from django.test import TestCase
from django.contrib.auth import get_user_model
from on_demand.models import UserDetails

class UserDetailsTest(TestCase):
  @classmethod
  def setUpTestData(cls):
    user = get_user_model().objects.create_user(username='testuser@example.com', email='testuser@example.com', password='secret')
    user = UserDetails.objects.get(id=1)
    user.picture = 'SomePictureURL'
    user.save(update_fields=['picture'])
    

  # Testing label for field defined
  def test_email_label(self):
    user = UserDetails.objects.get(id=1)
    email_label = user._meta.get_field('email').verbose_name
    self.assertEquals(email_label, 'email')
  
  # Testing email field is unique
  def test_email_is_unique(self):
    user = UserDetails.objects.get(id=1)
    email_uniqueness = user._meta.get_field('email').unique
    self.assertEquals(email_uniqueness, True)

  # Testing label for field defined
  def test_picture_label(self):
    user = UserDetails.objects.get(id=1)
    picture_label = user._meta.get_field('picture').blank
    self.assertEquals(picture_label, True)


import json
from django.test import TestCase
from django.contrib.auth import get_user_model
from on_demand.models import UserDetails, Connection
from on_demand.serializers import ConnectionSerializer


class ConnectionSerializerTest(TestCase):

  def test_connection_deserialization(self):
    supplier = get_user_model().objects.create_user(username='supplier@domain.com', email='supplier@domain.com', password='Supplier123')
    consumer = get_user_model().objects.create_user(username='consumer@domain.com', email='consumer@domain.com', password='Consumer123')
    connection = Connection.objects.create(supplier=supplier,consumer=consumer,status='requested')
    connection = Connection.objects.get(id=1)
    deserialized_connection = ConnectionSerializer(connection)
    expected_deserialized = "{'id': 1, 'supplier': 1, 'consumer': 2, 'status': 'requested', 'date_created': '" + connection.date_created.isoformat() +"', 'date_finished': None, 'objective': '', 'rejection_reason': None, 'consumer_request_comments': None, 'finish_reason': None, 'ranking': None}"
    self.assertEquals(expected_deserialized, str(deserialized_connection.data))
    


from django.test import TestCase

class StatusViewTest(TestCase):
  def test_status_url_exists_at_desired_location(self):
    response = self.client.get('/api/status/')
    self.assertEqual(response.status_code, 200)


# Generated by Django 2.1 on 2020-05-28 20:06

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Connection',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('status', models.TextField()),
                ('date_created', models.DateTimeField(default=django.utils.timezone.now)),
                ('date_finished', models.DateTimeField(blank=True, null=True)),
                ('objective', models.TextField()),
                ('rejection_reason', models.TextField(blank=True, null=True)),
                ('finish_reason', models.TextField(blank=True, null=True)),
                ('consumer_request_comments', models.TextField(blank=True, null=True)),
                ('ranking', models.IntegerField(blank=True, null=True)),
                ('consumer', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='connection_consumer', to=settings.AUTH_USER_MODEL)),
                ('supplier', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='connection_supplier', to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='ConsumerProfile',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('date_joined', models.DateTimeField(default=django.utils.timezone.now)),
                ('user', models.OneToOneField(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='consumer_profile', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'db_table': 'on_demand_consumer_profile',
            },
        ),
        migrations.CreateModel(
            name='SupplierProfile',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('finished_connections_count', models.IntegerField(default=0)),
                ('connections_ranking_accumulator', models.IntegerField(default=0)),
                ('date_joined', models.DateTimeField(default=django.utils.timezone.now)),
                ('user', models.OneToOneField(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='supplier_profile', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'db_table': 'on_demand_supplier_profile',
            },
        ),
        migrations.CreateModel(
            name='UserDetails',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('email', models.EmailField(max_length=254, unique=True)),
                ('picture', models.TextField(blank=True, null=True)),
                ('linkedin', models.TextField(blank=True, null=True)),
                ('behance', models.TextField(blank=True, null=True)),
                ('twitter', models.TextField(blank=True, null=True)),
                ('instagram', models.TextField(blank=True, null=True)),
                ('facebook', models.TextField(blank=True, null=True)),
                ('youtube', models.TextField(blank=True, null=True)),
                ('description', models.TextField(blank=True, null=True)),
                ('date_joined', models.DateTimeField(default=django.utils.timezone.now)),
                ('user', models.OneToOneField(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='details', to=settings.AUTH_USER_MODEL)),
            ],
        ),
    ]



from django.db import models
from django.contrib.auth.models import AbstractUser
from django.conf import settings
from django.utils import timezone
from django.contrib.auth.models import User

from django.db.models.signals import post_save
from django.dispatch import receiver


class UserDetails(models.Model):
    user = models.OneToOneField(
        settings.AUTH_USER_MODEL,
        on_delete=models.SET_NULL,
        related_name='details',
        null=True
    )
    email = models.EmailField(unique=True)
    picture = models.TextField(blank=True, null=True)
    linkedin = models.TextField(blank=True, null=True)
    behance = models.TextField(blank=True, null=True)
    twitter = models.TextField(blank=True, null=True)
    instagram = models.TextField(blank=True, null=True)
    facebook = models.TextField(blank=True, null=True)
    youtube = models.TextField(blank=True, null=True)
    description = models.TextField(blank=True, null=True)
    date_joined = models.DateTimeField(default=timezone.now)


class SupplierProfile(models.Model):
    user = models.OneToOneField(
        settings.AUTH_USER_MODEL,
        on_delete=models.SET_NULL,
        related_name='supplier_profile',
        null=True
    )
    finished_connections_count = models.IntegerField(default=0)
    connections_ranking_accumulator = models.IntegerField(default=0)
    date_joined = models.DateTimeField(default=timezone.now)

    class Meta:
        db_table = 'on_demand_supplier_profile'


class ConsumerProfile(models.Model):
    user = models.OneToOneField(
        settings.AUTH_USER_MODEL,
        on_delete=models.SET_NULL,
        related_name='consumer_profile',
        null=True
    )
    date_joined = models.DateTimeField(default=timezone.now)

    class Meta:
        db_table = 'on_demand_consumer_profile'


class Connection(models.Model):
    status = models.TextField()
    supplier = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        on_delete=models.CASCADE,
        related_name='connection_supplier')
    consumer = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        on_delete=models.CASCADE,
        related_name='connection_consumer')
    date_created = models.DateTimeField(default=timezone.now)
    date_finished = models.DateTimeField(blank=True, null=True)
    objective = models.TextField()
    rejection_reason = models.TextField(blank=True, null=True)
    finish_reason = models.TextField(blank=True, null=True)
    consumer_request_comments = models.TextField(blank=True, null=True)
    ranking = models.IntegerField(blank=True, null=True)



import django.dispatch

connection_requested = django.dispatch.Signal(providing_args=["from_user", "to_user"])

connection_cancelled = django.dispatch.Signal(providing_args=["from_user", "to_user", "role_to_notify", "connection_cancelled"])

connection_finished = django.dispatch.Signal(providing_args=["from_user", "to_user", "connection", "supplier_ranking"])

connection_accepted = django.dispatch.Signal(providing_args=["from_user", "to_user", "connection"])

connection_rejected = django.dispatch.Signal(providing_args=["from_user", "to_user", "connection", "rejection_reason"])